package com.community.huiju.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.dao.*;
import com.community.huiju.model.*;
import com.community.huiju.service.TaUserVerifyServicel;
import com.community.huiju.vo.TaUserVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 *
 * app端用户审核表，存放用户的角色还有房产信息
 * @author weiximei
 */
@Slf4j
@Service
public class TaUserVerifyServicelmpl implements TaUserVerifyServicel {

    @Autowired
    private TaUserVerifyMapper taUserVerifyMapper;

    @Autowired
    private ToCommunitiesMapper toCommunitiesMapper;

    @Autowired
    private TaSysRoleMapper taSysRoleMapper;

    @Autowired
    private SysNationMapper sysNationMapper;

    @Autowired
    private TaUserMapper taUserMapper;

    /**
     * 期
     */
    @Autowired
    private TpPhaseMapper tpPhaseMapper;

    /**
     * 单元
     */
    @Autowired
    private TpUnitMapper tpUnitMapper;

    /**
     * 楼层
     */
    @Autowired
    private TpLevelMapper tpLevelMapper;

    /**
     * 栋
     */
    @Autowired
    private TpBuildingMapper tpBuildingMapper;

    /**
     * 户号 /  楼栋资料库
     */
    @Autowired
    private TpBuildingOwnerInfoMapper tpBuildingOwnerInfoMapper;

    @Override
    public ResponseBean getToAuditNotApproved(UserElement userElement, Integer userVerifyId) {
        ResponseBean responseBean = new ResponseBean();
        TaUserVerify taUserVerify = taUserVerifyMapper.selectByPrimaryKey(userVerifyId);
        TaSysRole taSysRole = taSysRoleMapper.selectByPrimaryKey(taUserVerify.getRoleId());
        taUserVerify.setRoleName(taSysRole.getRoleName());
        if ("2".equals(taUserVerify.getVerifyStatus())) {

            // 开始查询 业主
            TpBuildingOwnerInfo tpBuildingOwnerInfo = tpBuildingOwnerInfoMapper.selectCommunityIdAndAddress(taUserVerify.getCommunityId(), taUserVerify.getPhaseId(), taUserVerify.getBuildingId(), taUserVerify.getUnitId(), taUserVerify.getLevelId(), taUserVerify.getRoomNoId());
            taUserVerify.setOwnerName(tpBuildingOwnerInfo.getOwnerName());
        }

        responseBean.addSuccess(taUserVerify);
        return responseBean;
    }

    @Override
    public ResponseBean addUserVerify(UserElement userElement, String parameter) {
        ResponseBean responseBean = new ResponseBean();

        JSONObject jsonObject = JSONObject.parseObject(parameter);
        TaUserVerify taUserVerify = jsonObject.toJavaObject(TaUserVerify.class);

        // 通过 角色名称 查询出角色的id
        String roleName = jsonObject.getString("roleName");
        TaSysRole sysRole = taSysRoleMapper.selectByRoleName(roleName);

        // 查询 期/栋/单元/楼层/户号 -> 设置 相应 id 和 name
        TpPhase phase = tpPhaseMapper.selectByPrimaryKey(taUserVerify.getPhaseId());
        TpBuilding building = tpBuildingMapper.selectByPrimaryKey(taUserVerify.getBuildingId());
        TpUnit unit = tpUnitMapper.selectByPrimaryKey(taUserVerify.getUnitId());
        TpLevel level = tpLevelMapper.selectByPrimaryKey(taUserVerify.getLevelId());
        TpBuildingOwnerInfo buildingOwnerInfo = tpBuildingOwnerInfoMapper.selectByPrimaryKey(taUserVerify.getRoomNoId());

        taUserVerify.setRoleId(sysRole.getId());
        taUserVerify.setCommunityId(userElement.getCommunityId());
        taUserVerify.setUserId(userElement.getId());
        taUserVerify.setVerifyStatus("0");
        taUserVerify.setCreateDate(new Date());
        taUserVerify.setPhaseName(phase.getName());
        taUserVerify.setBuildingName(building.getName());
        taUserVerify.setUnitName(unit.getName());
        taUserVerify.setLevelName(level.getName());
        taUserVerify.setRoomNoName(buildingOwnerInfo.getName());

        int rows = taUserVerifyMapper.insertSelective(taUserVerify);
        if (rows <= 0) {
            responseBean.addError("操作失败！");
            return responseBean;
        }

        responseBean.addSuccess(taUserVerify);
        return responseBean;
    }

    @Override
    public ResponseBean deleteUserVerify(Integer userVerifyId) {
        ResponseBean responseBean = new ResponseBean();

        TaUserVerify userVerify = taUserVerifyMapper.selectByPrimaryKey(userVerifyId);
        if ("1".equals(userVerify.getVerifyStatus())) {
            responseBean.addError("已审核通过的房产，无法删除！");
            return responseBean;
        }

        taUserVerifyMapper.deleteByPrimaryKey(userVerifyId);
        responseBean.addSuccess("操作成功！");
        return responseBean;
    }

    @Override
    public ResponseBean getByCurrentUser(UserElement userElement) {
        ResponseBean responseBean = new ResponseBean();

        List<TaUserVerify> userVerifyList = taUserVerifyMapper.selectByUserId(userElement.getId());
        userVerifyList.forEach(e -> {
            // 角色
            TaSysRole taSysRole = taSysRoleMapper.selectByPrimaryKey(e.getRoleId());
            e.setRoleName(taSysRole.getRoleName());
            // 小区名称 、省市区
            setUserVoProperties(e.getCommunityId(), e);
        });

        responseBean.addSuccess(userVerifyList);
        return responseBean;
    }

    /**
     * 设置 TaUserVO 的一些信息
     *
     *      小区，省市区
     *
     * @param communityId
     * @param taUserVerify
     */
    private void setUserVoProperties(Integer communityId, TaUserVerify taUserVerify) {
        // 获取小区
        ToCommunities toCommunities = toCommunitiesMapper.selectByPrimaryKey(communityId);
        // 省
        SysNation province = sysNationMapper.selectByPrimaryKey(toCommunities.getProvinceId());
        // 市
        SysNation city = sysNationMapper.selectByPrimaryKey(toCommunities.getCityId());
        // 区
        SysNation district = sysNationMapper.selectByPrimaryKey(toCommunities.getDistrictId());

        taUserVerify.setCommunityAddress(province.getProvince()+""+city.getCity()+""+district.getDistrict() + toCommunities.getCommunityName());
        taUserVerify.setCommunityName(toCommunities.getCommunityName());
    }

    @Override
    public ResponseBean deleteCurrentUserVerify(UserElement userElement, Integer userVerifyId) {
        ResponseBean responseBean = new ResponseBean();

        TaUserVerify userVerify = taUserVerifyMapper.selectByPrimaryKey(userVerifyId);
        if (null != userVerify && userElement.getUserVerifyId().intValue()  == userVerify.getId().intValue()) {
            responseBean.addError("无法删除当前登录的房产！");
            return responseBean;
        }
        if (null != userVerify && userElement.getId().intValue() == userVerify.getUserId().intValue()) {
            responseBean.addError("您名下无此房产关联！");
            return responseBean;
        }

        taUserVerifyMapper.deleteByPrimaryKey(userVerifyId);
        responseBean.addSuccess("操作成功！");
        return responseBean;
    }

    @Override
    public ResponseBean verifyUser(UserElement userElement, Integer userVerifyId, String userVerifyStatus) {
        ResponseBean responseBean = new ResponseBean();

        TaUserVerify userVerify = taUserVerifyMapper.selectByPrimaryKey(userVerifyId);
        if (null == userVerify) {
            responseBean.addError("该审核不存在！");
            return responseBean;
        }
        List<TaUser> userList = taUserMapper.selectHouseUserList(userElement.getCommunityId(), userVerify.getPhaseId(), userVerify.getBuildingId(), userVerify.getUnitId(), userVerify.getLevelId(), userVerify.getRoomNoId(), userElement.getId());
        boolean bool = userList.stream().anyMatch(e -> e.getId().intValue() == userVerify.getId().intValue());
        if (bool) {
            userVerify.setVerifyStatus(userVerifyStatus);
            taUserVerifyMapper.updateByPrimaryKeySelective(userVerify);
            responseBean.addSuccess("操作成功！");
            return responseBean;
        }
        responseBean.addError("操作失败！");
        return responseBean;
    }
}
