package com.community.huiju.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.commom.utils.AccountValidatorUtil;
import com.community.huiju.dao.TpBuildingMapper;
import com.community.huiju.dao.TpBuildingOwnerInfoMapper;
import com.community.huiju.dao.TpLevelMapper;
import com.community.huiju.dao.TpPhaseMapper;
import com.community.huiju.dao.TpUnitMapper;
import com.community.huiju.model.TaUser;
import com.community.huiju.model.TpBuilding;
import com.community.huiju.model.TpBuildingOwnerInfo;
import com.community.huiju.model.TpLevel;
import com.community.huiju.model.TpPhase;
import com.community.huiju.model.TpUnit;
import com.community.huiju.service.BuildingTreeServiceI;
import org.apache.ibatis.annotations.Mapper;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author FXF
 * @date 2019-04-01
 */
@Service("buildingTreeService")
public class BuildingTreeServiceImpl implements BuildingTreeServiceI {
	
	public static final Logger logger = LoggerFactory.getLogger(BuildingOwnerInfoServiceImpl.class);
	
	@Autowired
	private TpPhaseMapper tpPhaseMapper;
	
	@Autowired
	private TpBuildingMapper tpBuildingMapper;
	
	@Autowired
	private TpUnitMapper tpUnitMapper;
	
	@Autowired
	private TpLevelMapper tpLevelMapper;
	
	@Autowired
	private TpBuildingOwnerInfoMapper tpBuildingOwnerInfoMapper;
	
	/**
	 * 获取整个小区的基础数据
	 *
	 * @param file
	 * @return
	 */
	@Override
	public ResponseBean getExcelData(MultipartFile file) {
		ResponseBean responseBean = new ResponseBean();
		List<TpBuildingOwnerInfo> list = new ArrayList<>();
		if (file == null) {
			responseBean.addError("对象不能为空");
			return responseBean;
		}
		//获取文件的名字
		String originalFilename = file.getOriginalFilename();
		Workbook workbook = null;
		try {
			if (originalFilename.endsWith(Constant.SUFFIX_2003)) {
				workbook = new HSSFWorkbook(file.getInputStream());
			} else if (originalFilename.endsWith(Constant.SUFFIX_2007)) {
				workbook = new XSSFWorkbook(file.getInputStream());
			}
		} catch (Exception e) {
			logger.info(originalFilename);
			e.printStackTrace();
			responseBean.addError("格式错误");
			return responseBean;
		}
		if (workbook == null) {
			logger.info(originalFilename);
			responseBean.addError("格式错误");
			return responseBean;
		} else {
			List<String> temBuildingList = new ArrayList<String>();
			//获取一个sheet也就是一个工作簿
			Sheet sheet = workbook.getSheetAt(0);
			int lastRowNum = sheet.getLastRowNum();
			//从第一行开始第一行一般是标题
			for (int j = 3; j <= lastRowNum; j++) {
				Row row = sheet.getRow(j);
				TpBuildingOwnerInfo tpBuildingOwnerInfo = new TpBuildingOwnerInfo();
				String phase = row.getCell(0).getStringCellValue().trim();
				String building = row.getCell(1).getStringCellValue().trim();
				String unit = row.getCell(2).getStringCellValue().trim();
				String level = row.getCell(3).getStringCellValue().trim();
				String roomNo = row.getCell(4).getStringCellValue().trim();
				
				int currentRow = j+1;
				if (StringUtils.isEmpty(phase)){
					responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
					return responseBean;
				}
				if (StringUtils.isEmpty(building)){
					responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
					return responseBean;
				}
				if (StringUtils.isEmpty(unit)){
					responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
					return responseBean;
				}
				if (StringUtils.isEmpty(level)){
					responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
					return responseBean;
				}
				if (StringUtils.isEmpty(roomNo)){
					responseBean.addError("第" + currentRow + "行" + "期/区不能为空!");
					return responseBean;
				}
				
				String buildingInfo = phase + building + unit + level + roomNo;
				if (temBuildingList.contains(buildingInfo)){
					logger.info("存在重复房源:{}",buildingInfo);
					responseBean.addError("第"+ currentRow +"行存在重复房源");
					return responseBean;
				}
				
				//把数据塞入temList校验是否有重复的数据
				temBuildingList.add(buildingInfo);
				//构建数据
				tpBuildingOwnerInfo.setPhaseName(phase);
				tpBuildingOwnerInfo.setBuildingName(building);
				tpBuildingOwnerInfo.setUnitName(unit);
				tpBuildingOwnerInfo.setLevelName(level);
				tpBuildingOwnerInfo.setName(roomNo);
				list.add(tpBuildingOwnerInfo);
			}
		}
		//构建分页
		Map<String,Object> data = new HashMap<String, Object>();
		data.put("list",list);
		data.put("total",list.size());
		
		responseBean.addSuccess(data);
		return responseBean;
	}
	
	/**
	 * 上传小区的基础数据
	 *
	 * @param file
	 * @param userElement
	 * @return
	 */
	@Override
	public ResponseBean submitExcelData(MultipartFile file, UserElement userElement) {
		ResponseBean responseBean = new ResponseBean();
		responseBean = getExcelData(file);
		if (!responseBean.getCode().equals(Constant.RESPONSE_SUCCESS)){
			return responseBean;
		}
		//获取要上传的数据
		Map<String,Object> map = (Map<String, Object>) responseBean.getData();
		List<TpBuildingOwnerInfo> list = (List<TpBuildingOwnerInfo>) map.get("list");
		
		//入库
		Integer communityId = userElement.getCommunityId();
		
		//保存数据到表中--拆分数据
		saveBuildingTreeData(communityId,list);
		responseBean.addSuccess("success");
		return responseBean;
	}
	
	/**
	 * 获取树形数据
	 *
	 * @param communityId
	 * @param id
	 * @param treeType
	 * @return
	 */
	@Override
	public ResponseBean getTreeList(Integer communityId, Integer id, String treeType) {
		ResponseBean responseBean = new ResponseBean();
		List<Map<String,Object>> resultList = new ArrayList<>();
		//获取区/期树形数据
		if (treeType.equals("phase")){
			QueryWrapper<TpPhase> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("community_id",communityId);
			queryWrapper.orderByAsc("create_date");
			List<TpPhase> list = tpPhaseMapper.selectList(queryWrapper);
			list.stream().forEach(e -> {
				Map<String,Object> map = new HashMap<>();
				map.put("name",e.getName());
				map.put("id",e.getId());
				map.put("type","building");
				resultList.add(map);
			});
		}else if (treeType.equals("building")){
			QueryWrapper<TpBuilding> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("community_id",communityId);
			queryWrapper.eq("phase_id",id);
			queryWrapper.orderByAsc("create_date");
			List<TpBuilding> list = tpBuildingMapper.selectList(queryWrapper);
			list.stream().forEach(e -> {
				Map<String,Object> map = new HashMap<>();
				map.put("name",e.getName());
				map.put("id",e.getId());
				map.put("type","unit");
				resultList.add(map);
			});
		}else if (treeType.equals("unit")){
			QueryWrapper<TpUnit> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("community_id",communityId);
			queryWrapper.eq("building_id",id);
			queryWrapper.orderByAsc("create_date");
			List<TpUnit> list = tpUnitMapper.selectList(queryWrapper);
			list.stream().forEach(e -> {
				Map<String,Object> map = new HashMap<>();
				map.put("name",e.getName());
				map.put("id",e.getId());
				map.put("type","level");
				resultList.add(map);
			});
		}else if (treeType.equals("level")){
			QueryWrapper<TpLevel> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("community_id",communityId);
			queryWrapper.eq("unit_id",id);
			queryWrapper.orderByAsc("create_date");
			List<TpLevel> list = tpLevelMapper.selectList(queryWrapper);
			list.stream().forEach(e -> {
				Map<String,Object> map = new HashMap<>();
				map.put("name",e.getName());
				map.put("id",e.getId());
				map.put("type","roomNo");
				resultList.add(map);
			});
		}else if (treeType.equals("roomNo")){
			QueryWrapper<TpBuildingOwnerInfo> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("community_id",communityId);
			queryWrapper.eq("level_id",id);
			queryWrapper.orderByAsc("create_date");
			List<TpBuildingOwnerInfo> list = tpBuildingOwnerInfoMapper.selectList(queryWrapper);
			list.stream().forEach(e -> {
				Map<String,Object> map = new HashMap<>();
				map.put("name",e.getName());
				map.put("id",e.getId());
				map.put("type","end");
				map.put("leaf",true);
				resultList.add(map);
			});
		}
		responseBean.addSuccess(resultList);
		return responseBean;
	}
	
	private void saveBuildingTreeData(Integer communityId, List<TpBuildingOwnerInfo> list) {
		String lastPhaseName = "";
		String lastBuildingName = "";
		String lastUnitName = "";
		String lastLevelName = "";
		String lastRoomNoName = "";
		
		Integer lastPhaseId = null;
		Integer lastBuildingId = null;
		Integer lastUnitId = null;
		Integer lastLevelId = null;
		
		for (TpBuildingOwnerInfo tpBuildingOwnerInfo : list){
			//插入期/区数据
			if (!tpBuildingOwnerInfo.getPhaseName().equals(lastPhaseName)){
				TpPhase tpPhase = new TpPhase();
				tpPhase.setName(tpBuildingOwnerInfo.getPhaseName());
				tpPhase.setCommunityId(communityId);
				tpPhase.setCreateDate(LocalDateTime.now());
				tpPhaseMapper.insert(tpPhase);
				//重置Last
				lastPhaseName = tpBuildingOwnerInfo.getPhaseName();
				lastBuildingName = "";
				lastUnitName = "";
				lastLevelName = "";
				lastRoomNoName = "";
				lastPhaseId = tpPhase.getId();
			}
			//插入楼栋数据
			if (!tpBuildingOwnerInfo.getBuildingName().equals(lastBuildingName)){
				TpBuilding tpBuilding = new TpBuilding();
				tpBuilding.setCommunityId(communityId);
				tpBuilding.setPhaseId(lastPhaseId);
				tpBuilding.setPhaseName(lastPhaseName);
				tpBuilding.setName(tpBuildingOwnerInfo.getBuildingName());
				tpBuilding.setCreateDate(LocalDateTime.now());
				tpBuildingMapper.insert(tpBuilding);
				lastBuildingName = tpBuildingOwnerInfo.getBuildingName();
				lastBuildingId = tpBuilding.getId();
			}
			//插入单元信息
			if (!tpBuildingOwnerInfo.getUnitName().equals(lastUnitName)){
				TpUnit tpUnit = new TpUnit();
				tpUnit.setCommunityId(communityId);
				tpUnit.setPhaseId(lastPhaseId);
				tpUnit.setPhaseName(lastPhaseName);
				tpUnit.setBuildingId(lastBuildingId);
				tpUnit.setBuildingName(lastBuildingName);
				tpUnit.setName(tpBuildingOwnerInfo.getUnitName());
				tpUnit.setCreateDate(LocalDateTime.now());
				tpUnitMapper.insert(tpUnit);
				lastUnitName = tpBuildingOwnerInfo.getUnitName();
				lastUnitId = tpUnit.getId();
			}
			//插入楼层信息
			if (!tpBuildingOwnerInfo.getLevelName().equals(lastLevelName)){
				TpLevel tpLevel = new TpLevel();
				tpLevel.setCommunityId(communityId);
				tpLevel.setPhaseId(lastPhaseId);
				tpLevel.setPhaseName(lastPhaseName);
				tpLevel.setBuildingId(lastBuildingId);
				tpLevel.setBuildingName(lastBuildingName);
				tpLevel.setUnitId(lastUnitId);
				tpLevel.setUnitName(lastUnitName);
				tpLevel.setName(tpBuildingOwnerInfo.getLevelName());
				tpLevel.setCreateDate(LocalDateTime.now());
				tpLevelMapper.insert(tpLevel);
				lastLevelName = tpBuildingOwnerInfo.getLevelName();
				lastLevelId = tpLevel.getId();
			}
			//插入房间号信息
			if (!tpBuildingOwnerInfo.getName().equals(lastRoomNoName)){
				TpBuildingOwnerInfo tpBuildingOwnerInfoNew = new TpBuildingOwnerInfo();
				tpBuildingOwnerInfoNew.setCommunityId(communityId);
				tpBuildingOwnerInfoNew.setPhaseId(lastPhaseId);
				tpBuildingOwnerInfoNew.setPhaseName(lastPhaseName);
				tpBuildingOwnerInfoNew.setBuildingId(lastBuildingId);
				tpBuildingOwnerInfoNew.setBuildingName(lastBuildingName);
				tpBuildingOwnerInfoNew.setUnitId(lastUnitId);
				tpBuildingOwnerInfoNew.setUnitName(lastUnitName);
				tpBuildingOwnerInfoNew.setLevelId(lastLevelId);
				tpBuildingOwnerInfoNew.setLevelName(lastLevelName);
				tpBuildingOwnerInfoNew.setName(tpBuildingOwnerInfo.getName());
				tpBuildingOwnerInfoNew.setCreateDate(LocalDateTime.now());
				tpBuildingOwnerInfoMapper.insert(tpBuildingOwnerInfoNew);
			}
		}
	}
}
